
_ISR_STARTADDRESS	EQU		0x31B08000
top_of_stacks		EQU		0x31AE0000

;======================================================================================================
;	Fin = 12MHz,
;
;	MPLLout = (m x Fin)/(p x 2^s), m=MDIV, p=PDIV, s=SDIV, Fin=10~30MHz
;	(350,3,2)=350Mhz, (400,3,2)=400Mhz, (225,3,1)=450Mhz, (250,3,1)=500Mhz, (267,3,1)=534Mhz
;	(400,3,1)=800Mhz
;
;	EPLLout = ((m + K/2^16)xFin)/(p x 2^s), m=MDIV, p=PDIV, s=SDIV, K=KDIV, Fin=10~40MHz
;	(48,1,4)=36Mhz, (32,1,3)=48Mhz, (40,1,3)=60Mhz, (48,1,3)=72Mhz, (28,1,2)=84Mhz, (32,1,2)=96Mhz
;======================================================================================================
;	When you intend to change CLK setup, you should consider a few factors to operate S3C2443 correctly
;	1. Setup Startup_MPLL, 
;		Startup_Mdiv, 
;		Startup_Pdiv, 
;		Startup_Sdiv, 
;		Startup_ARMCLKdiv, 
;		Startup_PREdiv,
;		Startup_HCLKdiv,
;		Startup_PCLKdiv
;	2. Define mSDR or mDDR
;	3. Setup tRAS, tARFC, CAS, tRCD, tRP, REFCYC parameters according to HCLK, refer to table in this code
;=====================================================================================		
;In  	M   	P   	S    	MPLL       
;12	267	3	1	534.000 	45%~55%
;12	400	3	1	800.000 	40%~60%



Startup_MPLL		EQU		534000000;400000000

Startup_Mdiv		EQU 	267;400
Startup_Pdiv		EQU 	3
Startup_Sdiv		EQU 	1

Startup_ARMCLKdiv	EQU	0;1	;	0	 :	ARMCLK	= MPLL/1
							;	1	 :	ARMCLK	= MPLL/2
							;	2	 :	ARMCLK	= MPLL/3
							;	3	 :	ARMCLK	= MPLL/4
							;	5	 :	ARMCLK	= MPLL/6
							;	7	 :	ARMCLK	= MPLL/8
							;	13	 :	ARMCLK	= MPLL/12
							;	15	 :	ARMCLK	= MPLL/16

Startup_PREdiv		EQU	1;2	;	0 :	PREDIV_CLK	= MPLL
							;	1 :	PREDIV_CLK	= MPLL/2
							;	2 :	PREDIV_CLK	= MPLL/3
							;	3 :	PREDIV_CLK	= MPLL/4
										
							;  	PREdiv   HCLKdiv
Startup_HCLKdiv		EQU	1	;	0   :	HCLK	= PREDIV_CLK
							;	0x1 :	HCLK	= PREDIV_CLK/2
							;	0x3 :	HCLK	= PREDIV_CLK/4

Startup_PCLKdiv		EQU	1	;	0	 :	PCLK	= HCLK
							;	1	 :	PCLK	= HCLK/2

;======================================================================================
;	MEM config	[mSDR, mDDR, DDR2]
;======================================================================================

					GBLL	mDDR		; 256Mbit x16 x2ea = 512Mbit mem size
					
	IF :DEF: mSDR
;---------------------------------------
; BANKCFG register  : SDRAM configure
;---------------------------------------
RASBW0				EQU		2			;	RAS addr 00=11bit,01-12bit,10=13bit, 11=14bit
RASBW1				EQU		2			;	RAS addr 00=11bit,01-12bit,10=13bit, 11=14bit
CASBW0				EQU		1			;	CAS addr 00=8bit,01-9bit,10=10bit, 11=11bit
CASBW1				EQU		1			;	CAS addr 00=8bit,01-9bit,10=10bit, 11=11bit
ADDRCFG0			EQU		1			; 	addre configure
												;   00={BA,RAS,CAS}, 01={RAS,BA,CAS}
ADDRCFG1			EQU		1			; 	addre configure
												;   00={BA,RAS,CAS}, 01={RAS,BA,CAS}
MEMCFG				EQU		2			; 	Ext.Mem 000=SDR,010=MSDR,100=DDRz,110=MDDR,001=DDR2
BW					EQU		0			; 	Bus width 00=32bit,01=16bit

;---------------------------------------
; BANKCON1 register : SDRAM timing control
;---------------------------------------
BStop				EQU		0			;	read burst stop control
WBUF				EQU		1			;	write buffer control
AP					EQU		1			;	auto precharge control
PWRDN				EQU		0			;	power down mode
DQSDelay			EQU		4			;	DQS delay

;-----------------------[SDRAM]-------------------------------
;    HCLK		tRAS		tARFC		CAS		tRCD		RP		REFCYC
;---------------------------------------------------------------
;18MHz			0		1		3		0		0		140
;36MHz			1		2		3		0		0		280
; 100Mhz		5		7		3		1		1		780
; 110Mhz		5		7		3		2		2		858
; 120Mhz		6		7		3		2		2		936
; 133Mhz		6		10		3		2		2		1037
; 133.5Mhz		6		10		3		2		2		1041
; 135Mhz		6		10		3		2		2		1053
; 136Mhz		6		10		3		2		2		1060
; 136.5Mhz		6		10		3		2		2		1064
; 137.3Mhz		6		10		3		2		2		1070
; 138Mhz		6		10		3		2		2		1076
; 138.67Mhz		6		10		3		2		2		1081
; 140Mhz		6		11		3		3		3		1092 (out of spec)
; 141Mhz		6		11		3		3		3		1099
; 142Mhz		7		11		3		3		3		1107 (out of spec)
; 143.6Mhz		7		11		3		3		3		1120 (out of spec)		
;---------------------------------------------------------------

;---------------------------------------
; BANKCON2 register : SDRAM timing control (133MHz)
;---------------------------------------
tRAS				EQU		6			;	Row active time
tARFC				EQU		10			;	Row cycle time
CL					EQU		3			;	CAS latency control
tRCD				EQU		2			;	RAS to CAS delay
tRP					EQU		2			; 	Row pre-charge time


;---------------------------------------
; BANKCON3 register : SDRAM MRS/EMRS register
;---------------------------------------
BA_EMRS				EQU		2			;	BA : EMRS
DS					EQU		0			;	Driver strength
PASR				EQU		0			;	PASR
BA_MRS				EQU		0			;	BA : MRS
TM					EQU		0			; 	Test Mode - mode register set
CL_MRS				EQU		3			; 	CAS Latency

;---------------------------------------
; REFRESH register : SDRAM refresh register
;---------------------------------------
REFCYC				EQU		1037		; 	refresh cycle

	ELIF :DEF: mDDR

;---------------------------------------
; BANKCFG register  : mDDR SDRAM configure
;---------------------------------------
RASBW0				EQU		2			;	RAS addr 00=11bit,01-12bit,10=13bit, 11=14bit
RASBW1				EQU		2			;	RAS addr 00=11bit,01-12bit,10=13bit, 11=14bit
CASBW0				EQU		2			;	CAS addr 00=8bit,01-9bit,10=10bit, 11=11bit
CASBW1				EQU		2			;	CAS addr 00=8bit,01-9bit,10=10bit, 11=11bit
ADDRCFG0			EQU		1			; 	addre configure
									;   	00={BA,RAS,CAS}, 01={RAS,BA,CAS}
ADDRCFG1			EQU		1			; 	addre configure
												;   00={BA,RAS,CAS}, 01={RAS,BA,CAS}
MEMCFG				EQU		6			; 	Ext.Mem 000=SDR,010=MSDR,100=DDRz,110=MDDR,001=DDR2
BW					EQU		1			; 	Bus width 00=32bit,01=16bit

;---------------------------------------
; BANKCON1 register : mDDR SDRAM timing control
;---------------------------------------
BStop				EQU		0			;	read burst stop control
WBUF				EQU		1			;	write buffer control
AP					EQU		1			;	auto precharge control
PWRDN				EQU		1			;	power down mode
DQSDelay			EQU		3			;	DQS delay

;-----------------------[DDR2 SDRAM]--------------------------
;    HCLK		tRAS		tARFC		CAS		tRCD		tRP		REFCYC
;---------------------------------------------------------------
;18MHz			0		1		3		0		0		140
;36MHz			1		2		3		0		0		280
;100Mhz			4		7		3		2		2		780
;110Mhz			4		8		3		2		2		858
;120Mhz			5		9		3		2		2		936
;133Mhz			5		10		3		2		2		1037		
;---------------------------------------------------------------

;---------------------------------------
; BANKCON2 register : mDDR SDRAM timing control
;---------------------------------------
tRAS				EQU		6			;	Row active time
tARFC				EQU		10			;	Row cycle time
CL					EQU		3			;	CAS latency control
tRCD				EQU		2			;	RAS to CAS delay
tRP					EQU		2			; 	Row pre-charge time

;---------------------------------------
; BANKCON3 register : mDDR SDRAM MRS/EMRS register
;---------------------------------------
BA_EMRS				EQU		2			;	BA : EMRS
DS					EQU		0			;	Driver strength
PASR				EQU		0			;	PASR
BA_MRS				EQU		0			;	BA : MRS
TM					EQU		0			; 	Test Mode - mode register set
CL_MRS				EQU		3			; 	CAS Latency

;---------------------------------------
; REFRESH register : mDDR SDRAM refresh register
;---------------------------------------
REFCYC				EQU		1037			; 	refresh cycle
;---------------------------------------------------------------------------

	ELIF :DEF: DDR2

;---------------------------------------
; BANKCFG register  : DDR2 SDRAM configure
;---------------------------------------
RASBW0				EQU		2			;	RAS addr 00=11bit,01-12bit,10=13bit, 11=14bit
RASBW1				EQU		2			;	RAS addr 00=11bit,01-12bit,10=13bit, 11=14bit
CASBW0				EQU		2			;	CAS addr 00=8bit,01-9bit,10=10bit, 11=11bit
CASBW1				EQU		2			;	CAS addr 00=8bit,01-9bit,10=10bit, 11=11bit
ADDRCFG0			EQU		1			; 	addre configure
												;   00={BA,RAS,CAS}, 01={RAS,BA,CAS}
ADDRCFG1			EQU		1			; 	addre configure
												;   00={BA,RAS,CAS}, 01={RAS,BA,CAS}
MEMCFG				EQU		1			; 	Ext.Mem 000=SDR,010=MSDR,100=DDRz,110=MDDR,001=DDR2
BW					EQU		1			; 	Bus width 00=32bit,01=16bit


;---------------------------------------
; BANKCON1 register : DDR2 SDRAM timing control
;---------------------------------------
PADLOAD				EQU		1
BStop				EQU		0			;	read burst stop control
WBUF				EQU		1			;	write buffer control
AP					EQU		0			;	auto precharge control
PWRDN				EQU		0			;	power down mode
DQSDelay			EQU		4			;	DQS delay

;-----------------------[DDR2 SDRAM]--------------------------
;    HCLK		tRAS		tARFC		CAS		tRCD		tRP		REFCYC
;---------------------------------------------------------------
;18MHz			0		1		3		0		0		140
;36MHz			1		2		3		0		0		280
;100Mhz			4		7		3		2		2		780
;110Mhz			4		8		3		2		2		858
;120Mhz			5		9		3		2		2		936
;133Mhz			5		10		3		2		2		1037		
;---------------------------------------------------------------

;---------------------------------------
; BANKCON2 register : DDR2 SDRAM timing control
;---------------------------------------
tRAS				EQU		5			;	Row active time
tARFC				EQU		13			;	Row cycle time
CL					EQU		3			;	CAS latency control
tRCD				EQU		1			;	RAS to CAS delay
tRP					EQU		1			; 	Row pre-charge time


;---------------------------------------
; BANKCON3 register : DDR2 SDRAM MRS/EMRS register
;---------------------------------------
BA_EMRS1			EQU		1			;	BA : EMRS
DLL_DIS				EQU		1
DLL_EN				EQU		0
nDQS_DIS			EQU		1
RDQS_DIS			EQU		0
OCD_MODE_EXIT		EQU		0
OCD_MODE_DEFAULT	EQU		7
BA_EMRS2			EQU		2			;	BA : EMRS
BA_EMRS3			EQU		3			;	BA : EMRS
DS					EQU		0			;	Driver strength
PASR				EQU		0			;	PASR
BA_MRS				EQU		0			;	BA : MRS
TM					EQU		0			; 	Test Mode - mode register set
CL_MRS				EQU		3			; 	CAS Latency
DLL_RESET_HIGH		EQU		1
DLL_RESET_LOW		EQU		0

;---------------------------------------
; REFRESH register : DDR2 SDRAM refresh register
;---------------------------------------
REFCYC				EQU		1037			; 	refresh cycle
;---------------------------------------------------------------------------
					
	ENDIF


;---------------------------------------
; nGCS0 = AMD Flash = Bank0 Controller Parameter setting
;---------------------------------------
IDCY0				EQU		0x0			;	Idle or turnaround cycles IDCY*HCLK
WSTRD0				EQU		0xe			;	Read wait state = tacc
WSTWR0				EQU		0xe			;	wrie wait state
WSTOEN0				EQU		0			; 	output enable assertion delay from CS
WSTWEN0				EQU		0			; 	write enable assertion delay

BlWriteEn			EQU		1			;bit21-SMBAA signal control:0-1at all times, 1 active for sync
AddrValidWriteEn	EQU		1			;bit20-SMADDRVALD during write:0-always high,1-active for write 	
BurstLenWrite		EQU		0			;bit1819-burst transfer length:0-4,1-8,3-continu(sync only) 	
SyncWriteDev		EQU		0			;bit17-0:async, 1:sync 	
BMWrite				EQU		0			;bit16-burt mode write : 0-non-burst, 1-burst 	
WrapRead			EQU		0			;bit14-0-disable, 1 enable
BlReadEn			EQU		1			;bit13-SMBAA signal :0-1 at all time, 1-active for sync read
AddrValidReadEn		EQU		1			;bit12-SMADDRVALID signal: 0-always HIGH, 1-active for async & sync read
BurstLenRead		EQU		0			;bit1011-burst transfer length:0-4,1-8,2-16,3-cont(sync only)
SyncReadDev			EQU		0			;bit9-sync access :0-async, 1-sync
BMRead				EQU		0			;bit8-burst mode red and async page mode
SMBLSPOL			EQU		0			;bit6-polarit of signal nSMBLS
MW					EQU		1			;bit45-memory width : 00-8bit,01-16bit,10-32bit
WP					EQU		0			;bit3-write protect
WaitEn				EQU		0			;bit2-external wait signal enable
WaitPol				EQU		0			;bit1-polarity of the external wait input for actiation
RBLE				EQU		0			;bit0-read byte lane enable
SMBCR0_0			EQU		((BMRead<<8)+(SMBLSPOL<<6)+(MW<<4)+(WP<<3)+(WaitEn<<2)+(WaitPol<<1)+RBLE)
SMBCR0_1			EQU		((WrapRead<<14)+(BlReadEn<<13)+(AddrValidReadEn<<12)+(BurstLenRead<<10)+(SyncReadDev<<9))
SMBCR0_2			EQU		((BlWriteEn<<21)+(AddrValidWriteEn<<20)+(BurstLenWrite<<18)+(SyncWriteDev<<17)+(BMWrite<<16))

WaitTourErr0		EQU		0			; external wait timeout error flag

WSTBRD0				EQU		0x1f			; burst read wait state

MemClkRatio			EQU		1			; SMMEMCLK :0-HCLK,1-HCLK/2,2-HCLK/3
SMClockEn			EQU		1			; SMCLK enable 0-only active during mem access,1-always running	


	END
